; docformat = 'rst'
;+
; This is an example program to demonstrate how to create a navigatable GOES-15 image plot
; with Coyote Graphics routines. The data files can be downloaded from here::
; 
;     image --    ftp://goes.gsfc.nasa.gov/pub/goeswest-lzw/overview2/ir4/
;     nav file -- ftp://goes.gsfc.nasa.gov/pub/goeswest-lzw/overview2/maps/
;
; :Categories:
;    Graphics
;    
; :Examples:
;    Save the program as "goes_15_plot.pro" and run it like this::
;       IDL> .RUN goes_15_plot
;       
; :Author:
;    FANNING SOFTWARE CONSULTING::
;       David W. Fanning 
;       1645 Sheely Drive
;       Fort Collins, CO 80526 USA
;       Phone: 970-221-0438
;       E-mail: david@idlcoyote.com
;       Coyote's Guide to IDL Programming: http://www.idlcoyote.com
;
; :History:
;     Change History::
;        Written, 23 January 2013 by David W. Fanning.
;
; :Copyright:
;     Copyright (c) 2013, Fanning Software Consulting, Inc.
;-
PRO GOES_15_Plot
   
   ; Set up variables for the plot. Normally, these values would be 
   ; passed into the program as positional and keyword parameters.
    navFile = 'overview2_0801031800_G11I04_L_int.nav'
    imgFile = '1301310000eG15G15I04.tif'
    
    ; Read the image file.
    image = Read_Tiff(imgFile)
    
    ; TIFF files always have to be reversed in Y direction.
    image = Reverse(image, 2)
    dims = Size(image, /Dimensions)
    
    ; Open the navigation file to get the number or rows and columns.
    OpenR, lun, navFile, /Get_Lun
    cols = 0L
    rows = 0L
    ReadU, lun, cols, rows
    
    ; Read the latitude and longitude values from the navigation file.
    data = IntArr(cols*rows*2)
    ReadU, lun, data
    Free_Lun, lun
    
    ; Reform the lat/lon vectors to match the image.
    data = Reform(data, 2, cols*rows)
    lats = Reform(data[0,*], cols, rows)
    lons = Reform(data[1,*], cols, rows)
    Undefine, data
    
    ; Reverse lats in Y direction.
    lats = Reverse(lats, 2)
    
    ; Make latitude and longitude vectors that go through the center of the image.
    latvec = Reform(lats[dims[0]/2, *])
    lonvec =        lons[*, dims[1]/2]
    
    ; Convert to floats and find the missing values.
    badLatIndices = Where(latvec EQ 9900, badLatCnt)
    latvec = latvec / 100.0
    latvec[badLatIndices] = !Values.F_NaN
    badLonIndices = Where(lonvec EQ 9900, badLonCnt)
    lonvec = lonvec / 100.0
    lonvec[badLonIndices] = !Values.F_NaN
    
    ; Find the min and max ranges.
    latmin = Min(latvec, /NaN, MAX=latmax)
    lonmin = Min(lonvec, /NaN, MAX=lonmax)
    
    ; Find the center of the image.
    centerLat = lats[dims[0]/2, dims[1]/2] / 100.0
    centerLon = lons[dims[0]/2, dims[1]/2] / 100.0
    
    ; Find the pixel locations where the image starts and stops.
    goodLatIndices = Where(Finite(latvec))
    latStart = Min(goodLatIndices)
    latEnd = Max(goodLatIndices)
    
    goodLonIndices = Where(Finite(lonvec))
    lonStart = Min(goodLonIndices)
    lonEnd = Max(goodLonIndices)
    
    ; Find the normalized position of the actual image in the window.
    x0 = lonStart / Float(dims[0])
    x1 = lonEnd / Float(dims[0])
    y0 = latStart / Float(dims[1])
    y1 = latEnd / Float(dims[1])
    
    ; Display the image.
    cgDisplay, Aspect=image, 600, 600
    cgLoadCT, 0
    TVLCT, cgColor('blu1', /Triple), 0
    
    scaled = BytScl(image, Top=254) + 1B
    
    ; Space pixels
    spaceIndices = Where(image LE 60, count)
    IF count GT 0 THEN scaled[spaceIndices] = 0
    
    cgImage, scaled
    ;cgPlotS, [x0, x0, x1, x1, x0], [y0, y1, y1, y0,y0], /Normal, color='yellow'
    
    ; Set up a satellite map projection. Limit=[left, top, right, bottom].
    cgMap_Set, centerLat, centerLon, /Satellite, Sat_P=[6.5, 0.0, 0.0], $
       Position=[x0, y0, x1, y1], /NoBorder, /NoErase, $
       Limit=[centerLat, lonvec[lonStart], latvec[latEnd], centerLon, centerLat, $
              lonvec[lonEnd], latvec[latStart], centerLon]
    
    ; Annotate the map.
    cgMap_Continents, /USA, /Coasts, Color='cornsilk'
    cgMap_Grid, /Label, Color='rose'
    
END

; This main program shows how to call the program and produce
; various types of output.

  ; Display the plot in a graphics window.
  GOES_15_Plot
  
  ; Display the plot in a resizeable graphics window.
  cgWindow, 'GOES_15_Plot', WXSize=600, WYSize=600, WAspect=1.0, $
    WTitle='GOES-15 Plot in a Resizeable Graphics Window'
  
  ; Create a PostScript file.
  cgPS_Open, 'goes_15_plot.ps', Landscape=0
  GOES_15_Plot
  cgPS_Close
  
  ; Create a PNG file.
  cgPS2Raster, 'goes_15_plot.ps', /PNG, /Portrait, Resize=30

END